unit Encode;

interface

uses
  SysUtils,
  Classes,
  Windows,
  Inifiles,
  ZLIBArchive2,
  StrUtils,
  SZCodeBaseX;

function Internal_FileEncode(SourceFileName: string; ScriptFile: TMemIniFile;
  InternalGroup: string): boolean; overload;

function Internal_FileEncode(SourceFilename: string;
  ScriptFileName, InternalGroup: string): boolean; overload;

function Internal_FileEncode(SourceFileNames: array of string;
  ScriptFileName, InternalGroup: string): boolean; overload;

function GetTempDir: string;

implementation

uses
  {$IFDEF PerformanceCounter}
  PerformanceCounter,
  {$ENDIF}
  OutputStreams;

function Internal_FileEncode(SourceFileNames: array of string;
  ScriptFileName, InternalGroup: string): boolean;
var
  IniFile: TMemIniFile;
  I: integer;
  B: boolean;
begin
  Result := False;
  if (not fileExists(ScriptFileName)) then
    exit;
  {$IFDEF PerformanceCounter}
  PC.StartMeasure('Loading INI into memory');
  {$ENDIF}
  IniFile := TMemIniFile.Create(ScriptFileName);
  try
    for I := Low(SourceFileNames) to High(SourceFileNames) do
    begin
      B := Internal_FileEncode(SourceFileNames[I], IniFile, InternalGroup);
      if not B then
        exit;
    end;
    IniFile.UpdateFile;
    Result := True;
  finally
    IniFile.Free;
  end;
end;

function Internal_FileEncode(SourceFileName: string;
  ScriptFileName, InternalGroup: string): boolean;
begin
  Result := Internal_FileEncode([SourceFileName], ScriptFileName, InternalGroup);
end;

function Internal_FileEncode(SourceFileName: string; ScriptFile: TMemIniFile;
  InternalGroup: string): boolean;
var
  t: string;
  SectionName, EncodedFileName: string;

  ArcTempFile:     TFileStream;
  ArcTempFileName: string;
  StringsOutputStream: TStringsOutputStream;

  iniSections: TStrings;
  iniStrings:  TStrings;

  ZFile: TZLBArc2;
begin
  Result := False;

  {$IFDEF PerformanceCounter}
  PC.StartMeasure('Initialization');
  {$ENDIF}

  EncodedFileName := ExtractFileName(SourceFileName);

  ArcTempFileName := GetTempDir + ChangeFileExt(EncodedFileName, '.arc');

  if (FileExists(ArcTempFileName)) then
    DeleteFile(PChar(ArcTempFileName));

  {$IFDEF PerformanceCounter}
  PC.StartMeasure('Compressing file');
  {$ENDIF}
  ZFile := TZlbArc2.Create();
  try
    Zfile.CreateArchive(ArcTempFileName);
    Zfile.AddFile(SourceFileName);
    ZFile.CloseArchive;

    {$IFDEF PerformanceCounter}
    PC.StartMeasure('Preparing INI file');
    {$ENDIF}

    SectionName := 'EncodedFile-' + InternalGroup + '-' + EncodedFileName;
    while ScriptFile.SectionExists(SectionName) do
      ScriptFile.EraseSection(SectionName);

    // Re-create the needed Ini-Section
    ScriptFile.WriteString(SectionName, 'lines', '-1');
    ScriptFile.UpdateFile;

    // read all sections
    iniSections := TStringList.Create;
    ScriptFile.ReadSections(iniSections);

    // get our needed section
    iniStrings := TStrings(iniSections.Objects[iniSections.IndexOf(SectionName)]);

    {$IFDEF PerformanceCounter}
    PC.StartMeasure('Reading archive and perform base64 encoding into IniFile');
    {$ENDIF}

    // Stream to stream
    ArcTempFile := TFileStream.Create(ArcTempFileName, fmOpenRead or fmShareDenyNone);

    StringsOutputStream := TStringsOutputStream.Create(iniStrings, 4090);

    // Encode the .arc file content directly via into
    // StringsOutputStream into iniStrings
    SZEncodeBase64(ArcTempFile, StringsOutputStream);
    StringsOutputStream.Finalize;

    t := Format('%d,%d', [ArcTempFile.Size, StringsOutputStream.TotalBytesWritten]);
    ScriptFile.WriteString(SectionName, 'lines',
      IntToStr(StringsOutputStream.LineCount - 1));

    FreeAndNil(ArcTempFile);
    FreeAndNil(StringsOutputStream);
    FreeAndNil(iniSections);

    ScriptFile.WriteString(InternalGroup, EncodedFileName, t);

    {$IFDEF PerformanceCounter}
    PC.StartMeasure('Cleanup');
    {$ENDIF}

    Result := True;
  finally
    ZFile.Free;
    if Assigned(ArcTempFile) then
      ArcTempFile.Free;
    if Assigned(StringsOutputStream) then
      StringsOutputStream.Free;
    if Assigned(iniSections) then
      iniSections.Free;
  end;
  {$IFDEF PerformanceCounter}
  PC.StopMeasure;
  {$ENDIF}
end;

function GetTempDir: string;
var
  Buffer: array[0..255] of char;
begin
  GetTempPath(SizeOf(Buffer) - 1, Buffer);
  Result := StrPas(Buffer);
end;

end.
